class Asset:

    def __init__(self, mv_sdk, base_url: str, domain: str, **kwargs: dict):
        """
        Initialize the Asset Domain
        """
        super(Asset, self)
        self.mv_sdk = mv_sdk
        self.base_url = base_url
        self.domain = domain
    
    def get(self, params=None, data=None, headers=None, auth=None, object_id=None,
            domain_id=None, domain_action=None, bulk=False):
        """

        """
        headers = headers or {}
        headers['Content-Type'] = 'application/x-www-form-urlencoded'

        return self.mv_sdk.request(
            'get',
            self.base_url,
            self.domain,
            params=params,
            data=data,
            headers=headers,
            auth=auth,
            object_id=object_id,
            domain_id=domain_id,
            domain_action=domain_action
        )

    def put(self, params=None, data=None, headers=None, auth=None, profile_id=None, domain_id=None, domain_action=None):
        """

        """
    
    # --------------
    # ASSET KEYWORDS
    # --------------
    
    def create_keywords(self, params=None, data=None, headers=None, auth=None, object_id=None, object_action='keywords',
                         domain_id=None, domain_action=None, bulk=None):
        """

        """
        headers = headers or {}
        headers['Accept'] = "application/json"
        headers['Content-Type'] = "application/json"
        headers['Accept-Encoding'] = "gzip, deflate, br"
        headers['Connection'] = "keep-alive"

        return self.mv_sdk.request(
            'post',
            self.base_url,
            self.domain,
            params=params,
            data=data,
            headers=headers,
            auth=auth,
            object_id=object_id,
            object_action=object_action,
            domain_id=domain_id,
            domain_action=domain_action,
            bulk=bulk
        )

    def delete_keyword(self, params=None, data=None, headers=None, auth=None, object_id=None, object_action='keyword',
                       domain_id=None, domain_action=None, bulk=None):
        """

        """
        headers = headers or {}
        headers['Content-Type'] = 'application/x-www-form-urlencoded'

        return self.mv_sdk.request(
            'delete',
            self.base_url,
            self.domain,
            params=params,
            data=data,
            headers=headers,
            auth=auth,
            object_id=object_id,
            object_action=object_action,
            domain_id=domain_id,
            domain_action=domain_action,
            bulk=bulk
        )
    
    def get_keywords(self, params=None, data=None, headers=None, auth=None, object_id=None, object_action='keywords',
                     domain_id=None, domain_action=None, bulk=None):
        """

        """
        headers = headers or {}
        headers['Content-Type'] = 'application/x-www-form-urlencoded'

        return self.mv_sdk.request(
            'get',
            self.base_url,
            self.domain,
            params=params,
            data=data,
            headers=headers,
            auth=auth,
            object_id=object_id,
            object_action=object_action,
            domain_id=domain_id,
            domain_action=domain_action,
            bulk=bulk
        )



class Bulk:

    def __init__(self, mv_sdk, base_url: str, domain: str, **kwargs: dict):
        """
        Initialize the Asset Domain
        """
        super(Bulk, self)
        self.mv_sdk = mv_sdk
        self.base_url = base_url
        self.domain = domain
    
    def post(self, params=None, data=None, headers=None, auth=None, object_id=None,
             domain_id=None, domain_action=None, bulk=False):
        """

        """
        headers = headers or {}

        return self.mv_sdk.request(
            'post',
            self.base_url,
            self.domain,
            params=params,
            data=data,
            headers=headers,
            auth=auth,
            object_id=object_id,
            domain_id=domain_id,
            domain_action=domain_action
        )



import requests
import json
import unittest
from unittest import mock
from urllib.parse import urlencode
import logging


def create_params(**kwargs):
    '''
    Used to create url parameters for API call
    '''
    path = kwargs.get("path")
    params = kwargs.get("params")
    if params:
        query_string = urlencode(eval(params))
    return f'{path}?{query_string}'

def mocked_requests_get(url, headers, data):
    class MockResponse:
        def __init__(self, url, headers, status_code):
            self.verb = "GET"
            self.url = url
            self.headers = headers
            self.data = data
            self.status_code = status_code

        def json(self):
            return {
                "verb": self.verb,
                "url": self.url,
                "headers": self.headers,
                "data": self.data
            }

    return MockResponse(url, headers, 200)

class APIRequester:
    '''
    Used to make the request
    '''
    def __init__(self, **kwargs):

        self.method = kwargs.get("method")
        self.url = kwargs.get("url")
        self.headers = kwargs.get("headers")
        self.data = kwargs.get("data")
    
    #@mock.patch('requests.get', side_effect=mocked_requests_get)
    #def get(self, mock_get):
    def get(self):
        logging.debug(f'Request:\nVerb: GET\nURL: {self.url}\nHeaders: {self.headers}\nData: {self.data}')
        response = requests.get(
                self.url,
                headers=self.headers,
                data=self.data
            )
        return response
    
    #@mock.patch('requests.post', side_effect=mocked_requests_get)
    #def post(self, mock_get):
    def post(self):
        logging.debug(f'Request:\nVerb:POST\nURL: {self.url}\nHeaders: {self.headers}\nData: {self.data}')
        response = requests.post(
                self.url,
                headers=self.headers,
                data=self.data
            )
        return response
    
    #@mock.patch('requests.post', side_effect=mocked_requests_get)
    #def post(self, mock_get):
    def delete(self):
        logging.debug(f'Request:\nVerb: DELETE\nURL: {self.url}\nHeaders: {self.headers}\nData: {self.data}')
        response = requests.delete(
                self.url,
                headers=self.headers,
                data=self.data
            )
        return response

class PathBuilder:
    '''
    Used to build the correct API path that includes
    parameters & filters
    '''
    def __init__(self, **kwargs):
        self.base_url = kwargs.get('base_url')
        self.domain = kwargs.get('domain')
        self.version = kwargs.get('version')
        self.object_id = kwargs.get('object_id')
        self.object_action = kwargs.get('object_action')
        self.domain_id = kwargs.get('domain_id')
        self.domain_action = kwargs.get('domain_action')
        self.params = kwargs.get('params')
        
    def build(self):
        paths = {
            "domains":{
                "asset": {
                    "path": 'assets'
                },
                "attribute": {
                    "path": 'attributes'
                },
                "branded_portal": {
                    "path": 'brandedportals'
                },
                "category": {
                    "path": 'categories'
                },
                "config": {
                    "path": 'config'
                },
                "connect": {
                    "path": 'connect'
                },
                "crop": {
                    "path": 'crop'
                },
                "direct_link": {
                    "path": 'directlinks'
                },
                "download": {
                    "path": 'downloads'
                },
                "home": {
                    "path": ''
                },
                "introduction_and_help": {
                    "path": 'introductionAndHelp'
                },
                "keyword_group": {
                    "path": 'keywordGroups'
                },
                "keyword": {
                    "path": 'keywords'
                },
                "notification": {
                    "path": 'notification'
                },
                "org_unit": {
                    "path": 'organizationalUnits'
                },
                "reports": {
                    "path": 'reports'
                },
                "saved_search": {
                    "path": 'savedsearches'
                },
                "search": {
                    "path": 'search'
                },
                "share": {
                    "path": 'share'
                },
                "public": {
                    "path": 'public'
                },
                "upload": {
                    "path": 'uploads'
                },
                "user_group": {
                    "path": 'groups'
                },
                "user": {
                    "path": 'users'
                }
            }
        }
        domain_info = paths['domains'][self.domain]
        sections = [domain_info['path']]

        if self.domain_action:
            sections.append(self.domain_action)
        if self.object_id:
            sections.append(self.object_id)
        if self.object_action:
            sections.append(self.object_action)
        
        
        uri = f'/{"/".join(sections)}'
        
        #manage params and filtering
        params = {}
        for param in self.params.keys():
            params[param] = self.params[param]
        if params:
            uri = create_params(params=json.dumps(params), path=uri)

        url = f'https://{self.base_url}{uri}'

        return [uri, url]


import os
import logging

from mvsdk.api import PathBuilder, APIRequester


class Client(object):
    """ 
    A client for accessing the MVAPI. 
    """
    def __init__(self):

        self.base_url = 'mv-api-whistler.mediavalet.com'
        self.auth_url = 'iam-qa.mediavalet.com'
        
        # Domains
        self._asset = None
        self._bulk = None
        self._connect = None
        self._keyword = None



    def request(self, method, base_url, domain, object_id=None,
                object_action=None, domain_id=None, domain_action=None,
                params=None, data=None, headers=None, auth=None, bulk=None):

        headers = headers or {}
        params = params or {}
        data = data or {}
        method = method.upper()
        bulk = bulk or False

        headers['User-Agent'] = 'MediaValetSDK/0.0.4'
        headers['Host'] = base_url or self.base_url
        if auth:
            headers['Authorization'] = f'Bearer {auth}'

        uri, url = PathBuilder(base_url=base_url, domain=domain, object_id=object_id,
                               object_action=object_action, domain_id=domain_id,
                               domain_action=domain_action, params=params).build()

        if bulk:
            return {
                'method': method,
                'uri': uri,
                'headers': headers,
                'data': data
            }
        
        api = APIRequester(url=url, headers=headers, data=data)
        
        if method == 'GET':
            response = api.get()
        elif method == 'POST':
            response = api.post()
        elif method == 'DELETE':
            response = api.delete()
        else:
            response = {'status_code': "405", 'json': "Verb not allowed"}

        logging.debug(response.headers)
        logging.debug(response.text)
        
        if response.status_code == 200:
            return {
                "status": response.status_code,
                "json": response.json()
            }
        else:
            return {
                "status": response.status_code,
                "json": {}
            }
         
    @property
    def asset(self):
        """
        Access the MVAPI Asset API
        """
        if self._asset is None:
            from mvsdk.rest.asset import Asset
            self._asset = Asset(self, self.base_url, 'asset')
        return self._asset
    
    @property
    def bulk(self):
        """
        Access the MVAPI Bulk API
        """
        if self._bulk is None:
            from mvsdk.rest.bulk import Bulk
            self._bulk = Bulk(self, self.auth_url, 'bulk')
        return self._bulk
    
    @property
    def connect(self):
        """
        Access the MVAPI Connect API
        """
        if self._connect is None:
            from mvsdk.rest.connect import Connect
            self._connect = Connect(self, self.auth_url, 'connect')
        return self._connect
    
    @property
    def keyword(self):
        """
        Access the MVAPI Keyword API
        """
        if self._keyword is None:
            from mvsdk.rest.keyword import Keyword
            self._keyword = Keyword(self, self.base_url, 'keyword')
        return self._keyword


import base64

class Connect:

    def __init__(self, mv_sdk, base_url: str, domain: str, **kwargs: dict):
        """
        Initialize the Asset Domain
        """
        super(Connect, self)
        self.mv_sdk = mv_sdk
        self.base_url = base_url
        self.domain = domain

    def auth(self, params=None, data=None, headers=None, auth=None, object_id=None, domain_id=None, domain_action=None):
        """
        """
        headers = headers or {}

        if auth:
            auth_string = base64.b64encode(
                bytes(f'{auth["client_id"]}:{auth["client_secret"]}', 'utf-8"')
                ).decode("utf-8")
            headers['Authorization'] = f'Basic {auth_string}'

        
        headers = headers or {}

        headers['Content-Type'] = "application/x-www-form-urlencoded"
        headers['Accept'] = '*/*'

        return self.mv_sdk.request(
            'post',
            self.base_url,
            self.domain,
            params=params,
            data=data,
            headers=headers,
            object_id=object_id,
            domain_id=domain_id,
            domain_action='token'
        )


class Keyword:

    def __init__(self, mv_sdk, base_url: str, domain: str, **kwargs: dict):
        """
        Initialize the Keyword Domain
        """
        super(Keyword, self)
        self.mv_sdk = mv_sdk
        self.base_url = base_url
        self.domain = domain

    def create(self, params=None, data=None, headers=None, auth=None, object_id=None, object_action=None, 
            domain_id=None, domain_action=None):
        """
        https://docs.mediavalet.com/#38562514-52f6-457e-9b7a-25486661112c
        """
        headers = headers or {}
        headers['Content-Type'] = 'application/x-www-form-urlencoded'

        return self.mv_sdk.request(
            'post',
            self.base_url,
            self.domain,
            params=params,
            data=data,
            headers=headers,
            auth=auth,
            object_id=object_id,
            object_action=object_action,
            domain_id=domain_id,
            domain_action=domain_action
        )

    def get(self, params=None, data=None, headers=None, auth=None, object_id=None, object_action=None, 
            domain_id=None, domain_action=None):
        """
        https://docs.mediavalet.com/#1e6608b7-5d9a-4904-8ffc-731fc6c4e9c3
        """
        headers = headers or {}
        headers['Content-Type'] = 'application/x-www-form-urlencoded'

        return self.mv_sdk.request(
            'get',
            self.base_url,
            self.domain,
            params=params,
            data=data,
            headers=headers,
            auth=auth,
            object_id=object_id,
            object_action=object_action,
            domain_id=domain_id,
            domain_action=domain_action
        )


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='mvsdk',
    version='0.0.4-23009081015',
    author='James Armstrong',
    author_email='j@armstro.ca',
    description='MediaValet Python SDK',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/armstro-ca/mvsdk',
    project_urls = {
        "Bug Tracker": "https://github.com/armstro-ca/mvsdk/issues"
    },
    license='MIT',
    packages=setuptools.find_packages(where='src'),
    install_requires=['requests'],
    package_dir={'': 'src'},
)